@@ -11,12 +11,12 @@
 # under the License.
 
 import copy
+import io
+from unittest import mock
 
-import mock
 from openstack import exceptions as sdk_exc
 from osc_lib import exceptions as exc
 from osc_lib import utils
-import six
 
 from senlinclient.tests.unit.v1 import fakes
 from senlinclient.v1 import profile as osc_profile
@@ -264,7 +264,7 @@ class TestProfileDelete(TestProfile):
         self.assertEqual('Failed to delete 1 of the 2 specified profile(s).',
                          str(error))
 
-    @mock.patch('sys.stdin', spec=six.StringIO)
+    @mock.patch('sys.stdin', spec=io.StringIO)
     def test_profile_delete_prompt_yes(self, mock_stdin):
         arglist = ['my_profile']
         mock_stdin.isatty.return_value = True
@@ -277,7 +277,7 @@ class TestProfileDelete(TestProfile):
         self.mock_client.delete_profile.assert_called_with('my_profile',
                                                            False)
 
-    @mock.patch('sys.stdin', spec=six.StringIO)
+    @mock.patch('sys.stdin', spec=io.StringIO)
     def test_profile_delete_prompt_no(self, mock_stdin):
         arglist = ['my_profile']
         mock_stdin.isatty.return_value = True
